
import string

from Products.PageTemplates.PageTemplateFile import PageTemplateFile
from AccessControl import ClassSecurityInfo
from Globals import InitializeClass

from OFS import Image, SimpleItem

from Products.Silva.File import ZODBFile, FileSystemFile, TRANSMAP

from Products.Silva.helpers import add_and_edit
from Products.Silva import config

from interfaces import ILocalAsset
from LocalSilvaObject import LocalSilvaObject
from Errors import LocaleNotSupported

class LocalFile(LocalSilvaObject):
    """
    unlike locale images, file objects are assumed to contain content of
    a certain language, and only of this certain language (e.g. PDF downloads, etc)
    
    the language is the one this object is created for.
    
    therefore it has only one language, and thus only one title
    nevertheless this is implemented as a subclass of LocalSilvaObject
    for less typing ;-)
    
    if you need a File object in different languages,
    You need to create a different object for each language, sorry
    """
    # XXX maybe I should have mabe '_file' a local attribute instead
    
    meta_type = 'Silva Local File'

    # XXX maybe own "edit" tab, etc ?
    manage_options = SimpleItem.SimpleItem.manage_options + \
                     LocalSilvaObject.manage_options

    def get_available_languages(self):
        # only one language available
        return (self._default_language,)

    # same for the supported
    get_supported_languages = get_available_languages
    
    def create_language_variant(self, new_lang, old_lang):
        raise LocaleNotSupported, new_lang


    def HEAD(self, REQUEST, RESPONSE):
        " answer a HEAD request "
        RESPONSE.setHeader('Content-language',self._default_language)
        return self._file.HEAD(REQUEST,RESPONSE)


class LocalZODBFile(LocalFile, ZODBFile):
    __implements__ = (ILocalAsset,)

    def __init__(self, id, title, file, language):
        ZODBFile.__init__(self, id, title, file)
        del self._title
        LocalSilvaObject.__init__(self, id, title, language)

InitializeClass(LocalZODBFile)

class LocalFileSystemFile(LocalFile, FileSystemFile):
    __implements__ = (ILocalAsset,)

    def __init__(self, id, title, file, language, repository=None):
        FileSystemFile.__init__(self, id, title, file, repository)
        del self._title
        LocalSilvaObject.__init__(self, id, title, language)

InitializeClass(LocalFileSystemFile)


# FIXME:
# I do not want to do this; but the factory method is a complete
# cut & paste from the Silva.File.manage_addFile :(

manage_addLocalFileForm = PageTemplateFile(
    "www/localfileAdd", globals(), __name__='manage_addLocalFileForm')

def manage_addLocalFile(self, id='', title='', file=''):
    """Add a File
    """
    files_service = getattr(self.aq_parent, 'service_files', None)
    id, _title = Image.cookId(id, title, file)
    #    ^
    #    |  
    #    +-- The cooked title is not used for creating file objects.
    
    # Copy code from ExtFile, but we don't want a dependency per se:
    id = string.translate(id, TRANSMAP)
    language = self.get_selected_language()
    
    # Switch storage type:
    service_files = getattr(self.aq_inner, 'service_files', None)
    assert service_files is not None, "There is no service_files. " \
        "Refresh your silva root."
    assert service_files.meta_type == 'Silva Files Service', "Cannot reach "\
        "the files service, a `%s' is in the way" % (service_files.meta_type, )
    if service_files.useFSStorage():        
        object = LocalFileSystemFile(id, title, file, language,
            service_files.filesystem_path())
    else:
        object = LocalZODBFile(id, title, file, language)
    self._setObject(id, object)
    object = getattr(self, id)
    return object

