# the installation for the SilvaLocalContent product

from Products.Silva.install import add_fss_directory_view

from interfaces import ILocalRoot
import LocalDocument, LocalFolder, LocalPublication, LocalRoot
import LocalFile, LocalImage


def install(root):
    """install the view infrastructure for SilvaLocalContent.
    """
    if not ILocalRoot.isImplementedBy(root):
        raise TypeError, 'SilvaLocalContent should be only installed in a locale aware root'
    # create the core views from filesystem
    add_fss_directory_view(root.service_views, 'SilvaLocalContent', \
                           __file__, 'views')
    
    registerViews(root.service_view_registry)

    configureSecurity(root)

    
def uninstall(root):
    unregisterViews(root.service_view_registry)
    root.service_views.manage_delObjects(['SilvaLocalContent'])


def is_installed(root):
    return hasattr(root.service_views, 'SilvaLocalContent')

content_classes = [ LocalDocument.LocalDocument, ]
# assets: not yet
# asset_classes = [ LocalFile.LocalFile, LocalImage.LocalImage ]
asset_classes = [ ]
# content_classes = []
all_classes = content_classes + \
              asset_classes  + \
              [ LocalFolder.LocalFolder,
                LocalPublication.LocalPublication,
                LocalRoot.LocalRoot ] 

def registerViews(reg):
    """Register views on registry.
    """

    for klass in content_classes:
        name, class_name = klass.meta_type, klass.__name__
        reg.register('public', name,
                     [ 'public',  class_name])
        reg.register('add', name, 
                     [ 'add',  class_name])
        reg.register('edit', name, 
                     [ 'edit', 'VersionedContent', 'Local', class_name])

    for klass in asset_classes:
        name, class_name = klass.meta_type, klass.__name__
        reg.register('public', name,
                     [ 'public',  class_name[5:] ])
        reg.register('add', name, 
                     [ 'add',  class_name[5:], class_name ])
        reg.register('edit', name, 
                     [ 'edit', 'Asset', class_name[5:], class_name ])

    # containers are done extra, manually
    reg.register('public', LocalRoot.LocalRoot.meta_type,
                 [ 'public',  'Folder'])
    reg.register('edit', LocalRoot.LocalRoot.meta_type, 
                 [ 'edit', 'Container', 'Publication', 'LocalPublication'])

    reg.register('add', LocalPublication.LocalPublication.meta_type,
                 [ 'add', 'LocalPublication'])
    reg.register('public', LocalPublication.LocalPublication.meta_type,
                 [ 'public',  'Folder'])
    reg.register('edit', LocalPublication.LocalPublication.meta_type, 
                 [ 'edit', 'Container', 'Publication', 'LocalPublication'])

    reg.register('add', LocalFolder.LocalFolder.meta_type,
                 [ 'add', 'LocalFolder'])
    reg.register('public', LocalFolder.LocalFolder.meta_type,
                 [ 'public',  'Folder'])
    reg.register('edit', LocalFolder.LocalFolder.meta_type, 
                 [ 'edit', 'Container', 'LocalFolder'])
    


def unregisterViews(reg):
    """Unregister views on registry.
    """
    for klass in all_classes:
        name, class_name = klass.meta_type, klass.__name__
        for view in ('public', 'edit','add'):
            reg.unregister(view, name)
    
    


def configureSecurity(root):
    """Update security permissions to allow authores to create most of the content types here
    """

    all_editor = ['Manager', 'ChiefEditor', 'Editor' ]
    all_author = all_editor + ['Author']    
    add_author_permissions  = [ klass.meta_type for klass in content_classes ] \
                              + [ LocalFolder.LocalFolder.meta_type ]
    add_editor_permissions = [ LocalPublication.LocalPublication.meta_type ]

    for add_permission in add_author_permissions:
        root.manage_permission('Add %ss' % add_permission, all_author)    
    for add_permission in add_editor_permissions:
        root.manage_permission('Add %ss' % add_permission, all_editor)    

                                  


if __name__ == '__main__':
    print """This module is not an installer. You don't have to run it."""
