import unittest
import Zope

#from DateTime import DateTime

# from Products.ParsedXML.ParsedXML import ParsedXML
from Products.Silva.SilvaObject import SilvaObject
from OFS import SimpleItem
# from Products.Silva.Folder import Folder
# from Products.Silva.Publication import Publication

from Products.SilvaLocalContent.LocalSilvaObject import LocalSilvaObject
from Products.SilvaLocalContent.Errors import LocaleNotSupported, LocaleAlreadyExists

import Globals

from test_utils import test_request, assert_equal_elements

class LocalTestContent(LocalSilvaObject, SilvaObject, SimpleItem.SimpleItem):
    " dummy class to be able to test the LocalSilvaObject mixin "
    # we need to implement SimpleItem to have acquisition ...
    pass

class LocalSilvaObjectTestCase(unittest.TestCase):


    def assertEqualsSequence(self, seq1, seq2):
        return assert_equal_elements(seq1, seq2)            

    def setUp(self):
        get_transaction().begin()
        self.connection = Zope.DB.open()
        try:
            self.root = test_request(self.connection)

            self.acceptor = self.root.REQUEST.AcceptLanguage
            self.root.REQUEST['URL1'] = '/foo'

            self.root.manage_addProduct['SilvaLocalContent'].manage_addLocalRoot('root', 'Root', ['en', 'la'])
            self.sroot = self.root.root

            self.sroot._setObject('test',LocalTestContent('test',u'Title','en'))

            self.test = self.sroot.test

            # checkServices(self.sroot)

        except:
            self.tearDown()
            raise


    def test_title(self):

        self.assertEquals(u'Title', self.test.get_title())
        self.acceptor.set_language('la')
        # XXX there is no variant for it: what should we do here ?
        # ... deciding to return the default instead ...
        self.assertEquals(u'Title', self.test.get_title())

        # FIXME this should raise a "language not supported" or the like ?
        self.test.set_title(u'Novus Titulus')
        self.assertEquals(u'Novus Titulus', self.test.get_title())
        self.assertEquals(u'Root', self.sroot.get_title())

        self.acceptor.set_language('en')
        self.assertEquals(u'Title', self.test.get_title())

    def test_addLanguage(self):

        self.assertEqualsSequence(['en', 'la'], self.sroot.get_available_languages())
        self.assertEqualsSequence(['en', 'la'], self.sroot.get_supported_languages())

        self.assertEqualsSequence(['en'], self.test.get_available_languages())
        self.assertEqualsSequence(['en', 'la'], self.test.get_supported_languages())
        self.assertEquals('Title', self.test.get_title_editable())

        # copys the language from the default
        self.test.create_language_variant('la')
 
        self.assertEqualsSequence(['en','la'], self.test.get_available_languages())

        self.test.set_title(u'New Title')
        self.assertEquals('New Title', self.test.get_title_editable())
        self.acceptor.set_language('la')
        self.assertEquals('Title', self.test.get_title_editable())
        self.test.set_title(u'Novum titulum')
        self.assertEquals(u'Novum titulum', self.test.get_title_or_id())
        
        self.acceptor.set_language('en')
        self.assertEquals(u'New Title', self.test.get_title_or_id())


    def test_addExistingLanguage(self):
        # a few possible failures 

        # does already exist
        try:
            self.test.create_language_variant('en')
        except LocaleAlreadyExists:
            pass
        else:
            self.fail('this test should have failed')
            
        # old language does not exist
        try:
            self.test.create_language_variant('la', old_language='fr')
        except LocaleNotSupported:
            pass
        else:
            self.fail('this test should have failed')
                    
        # language not supported by container
        try:
            self.test.create_language_variant('fr')
        except LocaleNotSupported:
            pass
        else:
            self.fail('this test should have failed')



    def test_public_languages(self):
        # for a plain local silva object, the published languages
        # are the same as the avalable langauges,
        # and the approved languages are empty

        assert_equal_elements(self.test.get_available_languages(),
                              self.test.get_public_languages())
        assert_equal_elements((),
                              self.test.get_approved_languages())

        # this remains true after adding a new variant ...
        self.test.create_language_variant('la')

        assert_equal_elements(self.test.get_available_languages(),
                              self.test.get_public_languages())
        assert_equal_elements((),
                              self.test.get_approved_languages())


    def tearDown(self):
        get_transaction().abort()
        self.connection.close()

        
def test_suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(LocalSilvaObjectTestCase, 'test'))
    return suite        
